/**
 * Copyright  2002 Matt Griffith
 * Copyright  2004-09 Michael Flanakin
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to 
 * permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the 
 * Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
 * OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
using System;

using NArgs;

namespace UpdateVersion
{
	/// <summary>
	/// Represents the command line options for this application.
	/// </summary>
	internal class Options
	{
		#region | Fields |

		/// <summary>
		/// Describes the available command line options and their proper usage.
		/// </summary>
		public const string Usage = "usage:\nUpdateVersion [{-s,--startdate} Date]\n" +
		                             "[{-b,--build} Fixed | MonthDay | Increment | BuildDay | <standard DateTime format>]\n" +
		                             "[(-p,--pin) x.x.x.x]\n" +
		                             "[{-r,--revision} Fixed | Automatic | Increment]\n" +
									 "[{-i,--inputfile} <filename>]\n" +
		                             "[{-o,--outputfile} <filename>]\n" +
		                             "[{-v,--version} Assembly | File]\n" +
		                             "[{-w,--write} True | False]\n" ;

		private const string InvalidPinnedVersionError = "The version number specified for the pin option is not valid. Please provide a version number in the correct format.";

		private readonly DateOption _startDateOption;
		private readonly StringOption _buildOption;
		private readonly StringOption _pinOption;
		private readonly StringOption _revisionOption;
		private readonly StringOption _inputFile;
		private readonly StringOption _outputFile;
		private readonly StringOption _versionOption;
		private readonly BooleanOption _writeOption;

		#endregion


		#region | Initialization |

		/// <summary>
		/// Initializes a new instance of the Options class with the specified command
		/// line arguments.
		/// </summary>
		/// <param name="args">The arguments passed on the command line.</param>
		public Options(string[] args)
		{
			// initialize the options
			this._startDateOption = new DateOption("s", "startdate");
			this._buildOption = new StringOption("b", "build", "Fixed");
			this._pinOption = new StringOption("p", "pin", "0.0.0.0");
			this._revisionOption = new StringOption("r", "revision", "Automatic");
			this._inputFile = new StringOption("i", "inputfile");
			this._outputFile = new StringOption("o", "outputfile");
			this._versionOption = new StringOption("v", "version");
			this._writeOption = new BooleanOption("w", "write", false);

			// create a new command line parser and add our options
			var parser = new CmdLineParser();
			parser.AddOption(_startDateOption);
			parser.AddOption(_buildOption);
			parser.AddOption(_pinOption);
			parser.AddOption(_revisionOption);
			parser.AddOption(_inputFile);
			parser.AddOption(_outputFile);
			parser.AddOption(_versionOption);
			parser.AddOption(_writeOption);

			// try to parse our options
			parser.Parse(args);
			ValidatePinOption();
		}

		#endregion


		#region | Properties |

		/// <summary>
		/// Gets the StartDate specified on the command line.
		/// </summary>
		/// <value>Returns DateTime.MinValue if this option was not provided.</value>
		public DateTime StartDate
		{
			get { return this._startDateOption.IsMissing ? this._startDateOption.DefaultValue : this._startDateOption.Value; }
		}


		/// <summary>
		/// Gets the BuildNumberType specified on the command line.
		/// </summary>
		/// <value>
		/// Returns the default BuildNumberType if the build option 
		/// was not specified on the command line.
		/// </value>
		public BuildNumberType BuildNumberType
		{
			get
			{
				return this._buildOption.IsMissing ? ToBuildNumberType(this._buildOption.DefaultValue) : ToBuildNumberType(this._buildOption.Value);
			}
		}


		/// <summary>
		/// Gets the build number format, if the <see cref="BuildNumberType"/> is <see cref="UpdateVersion.BuildNumberType.DateTimeFormat"/>.
		/// </summary>
		/// <value>The build number format.</value>
		public string BuildNumberFormat
		{
			get
			{
				return (this._buildOption.IsMissing || BuildNumberType != BuildNumberType.DateTimeFormat) 
					? null : this._buildOption.Value;
			}
		}


		/// <summary>
		/// Gets the PinVersion specified on the command line.
		/// </summary>
		/// <value>
		/// Returns null if the pin option 
		/// was not specified on the command line.
		/// </value>
		public Version PinVersion
		{
			get { return this._pinOption.IsMissing ? null : new Version(this._pinOption.Value); }
		}

		/// <summary>
		/// Indicates whether the version number is pinned.
		/// </summary>
		/// <value>
		/// Returns true if the pin option was specified on the command line. Otherwise
		/// returns false.
		/// </value>
		public bool VersionIsPinned
		{
			get { return !this._pinOption.IsMissing; }
		}


		/// <summary>
		/// Gets the RevisionNumberType specified on the command line.
		/// </summary>
		/// <value>
		/// Returns the default RevisionNumberType if the revision option 
		/// was not specified on the command line.
		/// </value>
		public RevisionNumberType RevisionNumberType
		{
			get
			{
				return this._revisionOption.IsMissing
					? ToRevisionNumberType(this._revisionOption.DefaultValue)
					: ToRevisionNumberType(this._revisionOption.Value);
			}
		}


		/// <summary>
		/// Gets the revision number format, if the <see cref="RevisionNumberType"/> is <see cref="UpdateVersion.RevisionNumberType.DateTimeFormat"/>.
		/// </summary>
		/// <value>The revision number format.</value>
		public string RevisionNumberFormat
		{
			get
			{
				return (this._revisionOption.IsMissing || RevisionNumberType != RevisionNumberType.DateTimeFormat)
					? null : this._revisionOption.Value;
			}
		}


		/// <summary>
		/// Gets the inputfile specified on the command line.
		/// </summary>
		/// <value>
		/// Returns null if the inputfile was not specified on the command line.
		/// </value>
		public string InputFilename
		{
			get { return this._inputFile.IsMissing ? null : this._inputFile.Value; }
		}


		/// <summary>
		/// Gets the outputfile specified on the command line.
		/// </summary>
		/// <value>
		/// Returns null if the outputfile was not specified on the command line.
		/// </value>
		public string OutputFilename
		{
			get { return this._outputFile.IsMissing ? null : this._outputFile.Value; }
		}


		/// <summary>
		/// Gets the VersionType specified on the command line.
		/// </summary>
		/// <value>
		/// Returns the default VersionType if the version option 
		/// was not specified on the command line.
		/// </value>
		public VersionType VersionType
		{
			get
			{
				return this._versionOption.IsMissing
					? ToVersionType(this._versionOption.DefaultValue) : ToVersionType(this._versionOption.Value);
			}
		}


		/// <summary>
		/// Gets a value representing whether to write the new version to the output stream (not the output file).
		/// </summary>
		/// <value>
		/// Returns a value specifying whether to write the new version to the command line.
		/// </value>
		public bool WriteNewVersion
		{
			get { return this._writeOption.Value; }
		}

		#endregion


		#region | Methods [Private] |

		/// <summary>
		/// Private helper that converts a string to the equivilent BuildNumberType.
		/// </summary>
		/// <param name="buildNumberDescription">
		/// The string representing a BuildNumberType.
		/// </param>
		/// <returns>
		/// Returns the default BuildNumberType if the string is not a recognized BuildNumberType.
		/// </returns>
		private static BuildNumberType ToBuildNumberType(string buildNumberDescription)
		{
			if (String.Compare(buildNumberDescription, BuildNumberType.Increment.ToString(), StringComparison.OrdinalIgnoreCase) == 0)
			{
				return BuildNumberType.Increment;
			}
			if (String.Compare(buildNumberDescription, BuildNumberType.MonthDay.ToString(), StringComparison.OrdinalIgnoreCase) == 0)
			{
				return BuildNumberType.MonthDay;
			}
			if (String.Compare(buildNumberDescription, BuildNumberType.BuildDay.ToString(), StringComparison.OrdinalIgnoreCase) == 0)
			{
				return BuildNumberType.BuildDay;
			}
			if (String.Compare(buildNumberDescription, BuildNumberType.Fixed.ToString(), StringComparison.OrdinalIgnoreCase) == 0)
			{
				return BuildNumberType.Fixed;
			}
			return String.IsNullOrEmpty(buildNumberDescription) ? BuildNumberType.Fixed : BuildNumberType.DateTimeFormat;
		}


		/// <summary>
		/// Private helper that converts a string to the equivilent RevisionNumberType.
		/// </summary>
		/// <param name="revisionNumberDescription">The string representing a revision number.</param>
		/// <returns>Returns the default RevisionNumberType if the string is not a recognized RevisionNumberType.</returns>
		private static RevisionNumberType ToRevisionNumberType(string revisionNumberDescription)
		{
			if (String.Compare(revisionNumberDescription, RevisionNumberType.Increment.ToString(), StringComparison.OrdinalIgnoreCase) == 0)
			{
				return RevisionNumberType.Increment;
			}
			if (String.Compare(revisionNumberDescription, RevisionNumberType.Fixed.ToString(), StringComparison.OrdinalIgnoreCase) == 0)
			{
				return RevisionNumberType.Fixed;
			}
			if (String.Compare(revisionNumberDescription, RevisionNumberType.Automatic.ToString(), StringComparison.OrdinalIgnoreCase) == 0)
			{
				return RevisionNumberType.Automatic;
			}
			return String.IsNullOrEmpty(revisionNumberDescription) ? RevisionNumberType.Automatic : RevisionNumberType.DateTimeFormat;
		}


		/// <summary>
		/// Private helper that converts a string to the equivilent VersionType.
		/// </summary>
		/// <param name="versionDescription">The string representing a VersionType.</param>
		/// <returns>Returns the default VersionType if the string is not a recognized RevisionNumberType.</returns>
		private static VersionType ToVersionType(string versionDescription)
		{
			return versionDescription.StartsWith("f", StringComparison.OrdinalIgnoreCase) 
				? VersionType.File : VersionType.Assembly;
		}


		/// <summary>
		/// A private helper method that verifies the pin option specified on the commandlind.
		/// </summary>
		/// <exception cref="InvalidCastException">The pin option is not a valid <see cref="Version"/>.</exception>
		[System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1806:DoNotIgnoreMethodResults", MessageId = "System.Version", 
			Justification = "The <c>new Version()</c> call is being used strictly for validation purposes.")]
		private void ValidatePinOption()
		{
			if (this._pinOption.IsMissing)
			{
				return;
			}
			
			try
			{
				new Version(this._pinOption.Value);
			}
			catch (Exception e)
			{
				throw new InvalidCastException(InvalidPinnedVersionError, e);
			}
		}

		#endregion
	}
}