/**
 * Copyright  2002 Matt Griffith
 * Copyright  2004-09 Michael Flanakin
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to 
 * permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the 
 * Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
 * OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
using System;
using System.Globalization;
using System.Threading;

namespace UpdateVersion
{
	/// <summary>
	/// Represents a calculator that can calculate new version numbers.
	/// </summary>
	internal class VersionCalculator
	{
		private readonly Version _originalVersion;
		private DateTime _startDate = DateTime.MinValue;
		private BuildNumberType _buildNumberType = BuildNumberType.Fixed;


		/// <summary>
		/// Initializes a new VersionCalculator with the specified version.
		/// </summary>
		/// <param name="originalVersion">
		/// A string containing the major, minor, build, and 
		/// revision numbers, where each number is delimited with a period character ('.').
		/// </param>
		public VersionCalculator(string originalVersion) : this(new Version(originalVersion))
		{}


		/// <summary>
		/// Initializes a new VersionCalculator with the specified version.
		/// </summary>
		/// <param name="originalMajor">The original major version number.</param>
		/// <param name="originalMinor">The original minor version number.</param>
		/// <param name="originalBuild">The original build version number.</param>
		/// <param name="originalRevision">The original revision version number.</param>
		public VersionCalculator(int originalMajor, int originalMinor, int originalBuild, int originalRevision) :
			this(new Version(originalMajor, originalMinor, originalBuild, originalRevision))
		{}


		/// <summary>
		/// Initializes a new VersionCalculator with the specified version.
		/// </summary>
		/// <param name="originalVersion">The original version.</param>
		public VersionCalculator(Version originalVersion)
		{
			this.RevisionNumberType = RevisionNumberType.Automatic;
			this._originalVersion = originalVersion;
		}


		/// <summary>
		/// Gets the version number that the calculated version is based on.
		/// </summary>
		public Version OriginalVersion
		{
			get { return this._originalVersion; }
		}


		/// <summary>
		/// Gets the project start date used to calculate new build numbers when
		/// using the MonthDay algorithm. 
		/// </summary>
		public DateTime StartDate
		{
			get { return this._startDate; }
			set
			{
				if (DateTime.MinValue != value && value > DateTime.Now)
					throw new ArgumentOutOfRangeException("value", "The start date can not be after today's date.");

				if (DateTime.MinValue == value && BuildNumberType.MonthDay == this._buildNumberType)
					this._buildNumberType = BuildNumberType.Fixed;

				this._startDate = value;
			}
		}


		/// <summary>
		/// Gets the algorithm to use when calculating the build number.
		/// </summary>
		public BuildNumberType BuildNumberType
		{
			get { return this._buildNumberType; }
			set
			{
				if (DateTime.MinValue == this._startDate && BuildNumberType.MonthDay == value)
					throw new ArgumentException("You must set the StartDate before setting the BuildNumberType to MonthDay.");
				
				this._buildNumberType = value;
			}
		}


		/// <summary>
		/// Gets the algorithm to use when calculating the revision number.
		/// </summary>
		public RevisionNumberType RevisionNumberType { get; set; }


		/// <summary>
		/// Gets the new calculated version.
		/// </summary>
		public Version NewVersion
		{
			get
			{
				return new Version(
					this._originalVersion.Major, 
					this._originalVersion.Minor, 
					CalculateBuildNumber(), 
					CalculateRevisionNumber());
			}
		}


		public string BuildNumberFormat { get; set; }
		
		
		public string RevisionNumberFormat { get; set; }


		/// <summary>
		/// Private helper method that calculates the build number.
		/// </summary>
		/// <returns>The new calculated build number.</returns>
		private int CalculateBuildNumber()
		{
			switch(this._buildNumberType)
			{
				case BuildNumberType.Increment:
					return this._originalVersion.Build + 1;
				
				case BuildNumberType.MonthDay:
					var calendar = Thread.CurrentThread.CurrentCulture.Calendar;
					var months = ((calendar.GetYear(DateTime.Today) - calendar.GetYear(this._startDate)) * 12) 
						+ calendar.GetMonth(DateTime.Today) - calendar.GetMonth(this._startDate);
					var day = DateTime.Now.Day;
					return (months * 100) + day;
				
				case BuildNumberType.BuildDay:
					var d = DateTime.Now;
					return Int32.Parse(String.Format(CultureInfo.CurrentCulture, "{0}{1:000}", 
						(d.Year % 10), d.DayOfYear), CultureInfo.CurrentCulture);

				case BuildNumberType.DateTimeFormat:
					return Convert.ToInt32(DateTime.Now.ToString(BuildNumberFormat));

				default:
					return this._originalVersion.Build;
			}
		}


		/// <summary>
		/// Private helper that calculates the revision number.
		/// </summary>
		/// <returns>The new calculated revision number.</returns>
		private int CalculateRevisionNumber()
		{
			switch(RevisionNumberType)
			{
				case RevisionNumberType.Increment:
					return this._originalVersion.Revision + 1;

				case RevisionNumberType.Automatic:
					return (int)(DateTime.Now.Subtract(DateTime.Today).TotalSeconds / 10);

				case RevisionNumberType.DateTimeFormat:
					return Convert.ToInt32(DateTime.Now.ToString(RevisionNumberFormat));

				default:
					return this._originalVersion.Revision;
			}
		}
	}
}