/**
 * Copyright  2002 Matt Griffith
 * Copyright  2004-09 Michael Flanakin
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to 
 * permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the 
 * Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
 * OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
using System;
using System.Globalization;
using System.Text.RegularExpressions;

namespace UpdateVersion
{
	/// <summary>
	/// Searches for an AssemblyVersion attribute in an input string and updates the version
	/// number using the provided options.
	/// </summary>
	internal class VersionUpdater
	{
		/// <summary>
		/// A regex that matches strings like 'AssemblyVersion("1.0.0.1")'
		/// </summary>
		private static readonly Regex AssemblyVersionRegex = 
			new Regex("AssemblyVersion(?:Attribute)?\\(\\s*?\"(?<version>(?<major>[0-9]+)\\.(?<minor>[0-9]+)\\.(?<build>[0-9]+)\\.(?<revision>[0-9]+))\"\\s*?\\)");

		private static readonly Regex FileVersionRegex = 
			new Regex("AssemblyFileVersion(?:Attribute)?\\(\\s*?\"(?<version>(?<major>[0-9]+)\\.(?<minor>[0-9]+)\\.(?<build>[0-9]+)\\.(?<revision>[0-9]+))\"\\s*?\\)");


		private readonly string _input;
		private readonly string _output;
		private readonly Version _newVersion;
		private readonly Regex _activeRegex;
		private readonly string _replaceFormat;


		/// <summary>
		/// Initializes a new VersionUpdater instance.
		/// </summary>
		/// <param name="input">
		/// The input string containing the AssemblyVersion attribute
		/// to update.
		/// </param>
		/// <param name="options">The options to use for updating the version number.</param>
		public VersionUpdater(string input, Options options)
		{
			// validate params
			if (options == null)
			{
				throw new ArgumentNullException("options");
			}

			// save input and default output
			this._input = input;
			this._output = input;
			
			// save assembly/file version regex/format
			if (options.VersionType == VersionType.Assembly)
			{
				this._activeRegex = AssemblyVersionRegex;
				this._replaceFormat = "AssemblyVersion(\"{0}\")";
			}
			else
			{
				this._activeRegex = FileVersionRegex;
				this._replaceFormat = "AssemblyFileVersion(\"{0}\")";
			}

			try
			{
				var match = this._activeRegex.Match(input);
				if (null != match)
				{
					var inputVersion = match.Groups["version"].Value;

					if (String.IsNullOrEmpty(inputVersion))
					{
						// no version in input; warn user and set output to input
						Console.Error.WriteLine("Version not found in input. Output equals input.");
						this._output = input;
					}
					else
					{
						// if pinned, use pin version
						var calculator = options.VersionIsPinned 
							? new VersionCalculator(options.PinVersion) : new VersionCalculator(inputVersion);

						// set start date
						calculator.StartDate = options.StartDate;

						// if pin version build number is invalid
						calculator.BuildNumberType = (options.PinVersion == null || options.PinVersion.Build == -1) 
							? options.BuildNumberType : BuildNumberType.Fixed;
						calculator.BuildNumberFormat = options.BuildNumberFormat;

						// if pin version revision number is invalid
						calculator.RevisionNumberType = (options.PinVersion == null || options.PinVersion.Revision == -1)
							? options.RevisionNumberType : RevisionNumberType.Fixed;
						calculator.RevisionNumberFormat = options.RevisionNumberFormat;

						// save new version
						this._newVersion = calculator.NewVersion;

						// save replacement string
						var replacement = String.Format(CultureInfo.CurrentCulture, this._replaceFormat, NewVersion);

						var outputVersion = this._activeRegex.Replace(input, replacement, 1);
						this._output = outputVersion;
					}
				}
			}
			catch (Exception)
			{
				// there was a problem finding or updating the version number
				this._output = input;
				throw;
			}
		}


		/// <summary>
		/// Gets the input string that this VersionUpdater was based on.
		/// </summary>
		public string Input
		{
			get { return this._input; }
		}


		/// <summary>
		/// Gets the output string with the updated version number.
		/// </summary>
		public string Output
		{
			get { return this._output; }
		}

		
		/// <summary>
		/// Gets the updated version number.
		/// </summary>
		public Version NewVersion
		{
			get { return this._newVersion; }
		}
	}
}